@@ -1,50 +1,40 @@
 /*
- * This file is part of the KDE Baloo Project
- * Copyright (C) 2012-2014  Vishesh Handa <me@vhanda.in>
- * Copyright (C) 2017-2018  James D. Smith <smithjd15@gmail.com>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) version 3, or any
- * later version accepted by the membership of KDE e.V. (or its
- * successor approved by the membership of KDE e.V.), which shall
- * act as a proxy defined in Section 6 of version 3 of the license.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
- *
- */
+    This file is part of the KDE Baloo Project
+    SPDX-FileCopyrightText: 2012-2014 Vishesh Handa <me@vhanda.in>
+    SPDX-FileCopyrightText: 2017-2018 James D. Smith <smithjd15@gmail.com>
+    SPDX-FileCopyrightText: 2020 Stefan Brüns <bruns@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
+*/
 
 #include "kio_tags.h"
 #include "kio_tags_debug.h"
 
 #include <QUrl>
-#include <QDebug>
 
 #include <KLocalizedString>
 #include <KUser>
-#include <KFileMetaData/UserMetaData>
 #include <KIO/Job>
 
 #include <QCoreApplication>
-#include <QDateTime>
 #include <QDir>
 #include <QRegularExpression>
 
 #include "file.h"
 #include "taglistjob.h"
+#include "../common/udstools.h"
 
-#include "query.h"
 #include "term.h"
 
 using namespace Baloo;
 
+// Pseudo plugin class to embed meta data
+class KIOPluginForMetaData : public QObject
+{
+    Q_OBJECT
+    Q_PLUGIN_METADATA(IID "org.kde.kio.slave.tags" FILE "tags.json")
+};
+
 TagsProtocol::TagsProtocol(const QByteArray& pool_socket, const QByteArray& app_socket)
     : KIO::ForwardingSlaveBase("tags", pool_socket, app_socket)
 {
@@ -84,7 +74,7 @@ void TagsProtocol::stat(const QUrl& url)
             ForwardingSlaveBase::stat(result.fileUrl);
             return;
         case TagUrl:
-            for (const KIO::UDSEntry& entry : qAsConst(result.pathUDSResults)) {
+            for (const KIO::UDSEntry& entry : std::as_const(result.pathUDSResults)) {
                 if (entry.stringValue(KIO::UDSEntry::UDS_EXTRA) == result.tag) {
                     statEntry(entry);
                 }
@@ -295,7 +285,7 @@ TagsProtocol::ParseResult TagsProtocol::parseUrl(const QUrl& url, const QList<Pa
     TagsProtocol::ParseResult result;
     result.decodedUrl = QUrl::fromPercentEncoding(url.toString().toUtf8());
 
-    if ((url.scheme() == QLatin1String("tags")) && result.decodedUrl.length()>6 && result.decodedUrl.at(6) == QChar('/')) {
+    if ((url.scheme() == QLatin1String("tags")) && result.decodedUrl.length()>6 && result.decodedUrl.at(6) == QLatin1Char('/')) {
         result.urlType = InvalidUrl;
         return result;
     }
@@ -323,7 +313,7 @@ TagsProtocol::ParseResult TagsProtocol::parseUrl(const QUrl& url, const QList<Pa
         uds.fastInsert(KIO::UDSEntry::UDS_DISPLAY_TYPE, displayType);
 
         QString displayName = i18n("All Tags");
-        if (!tag.isEmpty() && ((tagSection == QStringLiteral(".")) || (tagSection == QStringLiteral("..")))) {
+        if (!tag.isEmpty() && ((tagSection == QLatin1Char('.')) || (tagSection == QLatin1String("..")))) {
             displayName = tag.section(QLatin1Char('/'), -1);
         } else {
             displayName = tagSection;
@@ -394,9 +384,9 @@ TagsProtocol::ParseResult TagsProtocol::parseUrl(const QUrl& url, const QList<Pa
         if (!result.tag.isEmpty()) {
             // Create a query to find files that may be in the operation's scope.
             QString query = result.tag;
-            query.prepend("tag:");
-            query.replace(' ', " AND tag:");
-            query.replace('/', " AND tag:");
+            query.prepend(QStringLiteral("tag:"));
+            query.replace(QLatin1Char(' '), QStringLiteral(" AND tag:"));
+            query.replace(QLatin1Char('/'), QStringLiteral(" AND tag:"));
             result.query.setSearchString(query);
 
             qCDebug(KIO_TAGS) << result.decodedUrl << "url query:" << query;
@@ -444,31 +434,23 @@ TagsProtocol::ParseResult TagsProtocol::parseUrl(const QUrl& url, const QList<Pa
     q.setSearchString(QStringLiteral("tag=\"%1\"").arg(result.tag));
     ResultIterator it = q.exec();
     QList<QString> resultNames;
+    UdsFactory udsf;
+
     while (it.next()) {
-        const QUrl& match = QUrl::fromLocalFile(it.filePath());
+        KIO::UDSEntry uds = udsf.createUdsEntry(it.filePath());
+        if (uds.count() == 0) {
+	    continue;
+	}
 
-        // Somehow stat the file.
-        KIO::UDSEntry uds;
-        KIO::StatJob* job = KIO::stat(match, KIO::HideProgressInfo);
-        // We do not want to wait for the event loop to delete the job.
-        QScopedPointer<KIO::StatJob> sp(job);
-        job->setAutoDelete(false);
-        if (job->exec()) {
-            uds = job->statResult();
-        } else {
-            continue;
-        }
-
-        uds.fastInsert(KIO::UDSEntry::UDS_TARGET_URL, match.toString());
-        uds.fastInsert(KIO::UDSEntry::UDS_ICON_OVERLAY_NAMES, QStringLiteral("tag"));
-
-        if (resultNames.contains(match.fileName())) {
-            uds.replace(KIO::UDSEntry::UDS_NAME, match.fileName() + QStringLiteral(" (%1)").arg(resultNames.count(match.fileName())));
+	const QUrl url(uds.stringValue(KIO::UDSEntry::UDS_URL));
+	auto dupCount = resultNames.count(url.fileName());
+        if (dupCount > 0) {
+            uds.replace(KIO::UDSEntry::UDS_NAME, url.fileName() + QStringLiteral(" (%1)").arg(dupCount));
         }
 
         qCDebug(KIO_TAGS) << result.tag << "adding file:" << uds.stringValue(KIO::UDSEntry::UDS_NAME);
 
-        resultNames << match.fileName();
+        resultNames << url.fileName();
         result.pathUDSResults << uds;
     }
 
@@ -487,3 +469,4 @@ extern "C"
     }
 }
 
+#include "kio_tags.moc"
